#include "library.h"
#include <stdio.h>
#include <iostream>
#include <stdlib.h>
#include <iomanip>
#include <string>
#include <fstream>
#include <math.h>
#include <ctype.h>
#include <limits.h>
#include <assert.h>
#include <string.h>

int globalcount;
bool found1;
using namespace std;	

double g_lat = 0;
double g_long = 0;
int g_col = 0;
int g_rows = 0;
int g_sec = 0;				//Initialized to zero to prevent crashes
//
//
//
//
//-----STRUCTURES-----
struct road
	{
	string name1, type, length;
	void * destination;				//link road to intersection
	road * next;					//Create linked list within location info for pointer to all roads at intersection

	road(string n1, string t, string l, void*d=NULL, road*n2=NULL)
		{	
		name1 = n1;
		type = t; 
		length = l;
		destination = d;		
		next = n2;
		};
	road();

	void printroad();
	};
struct city				//Structure contains data for each city 
	{

	string abbrev, fips1, fips2, name, population, housing, 
	land1, water1, land2, water2, latitude, longitude,closest;	

	city(string a, string f1, string f2, string n, string p, string h, string l1, 
	string w1, string l2, string w2, string la, string lo, string cl);
	city();		

	void printall(city data);
	void draw_city (string direction);

	};	

struct coverage				//Choose proper map to draw based on starting and ending point
	{

	int startlat, endlat, startlong, endlong;
	string filename;

	coverage(int slat, int elat, int slong, int elong, string f);
	coverage();
	void printcov();

	};
struct location
	{

	string latitude, longitude, alphadist, population, type, abbrev, name; // alphadist is distance from nearest alphaplace
	road * r;
	location * destination;
	double distance;													// Used for shortest path, distance traveled so far

	location(string la, string lo, string al, string p, string t, string ab, string n, road * r1=NULL);
	location();

	void printloc();
	void printallroads(road*r,int count);

	};
class hashtable				//Includes N(used slots) and H(total slots) of entire hash table, structure for individual links, and important functions. 
	{
	protected:
	int N;
	int H;
	
	public:
	struct hashlink
		{
		city * data;					// Pointer to city data if hashlink is being used for city array
		location * data2;				// Pointer to location data if hashlink is being used for location array
		hashlink * next;
		hashlink * other;				// Used for pointing to a hashlink linking to a data type.  Ie if the hashlink is being used to point to an alphaplace, other points to its corresponding closest location.  
		hashlink(city*d=NULL,location*d2=NULL,hashlink*n=NULL,hashlink*o=NULL)
			{
			data=d;
			data2=d2;
			next=n;
			other=o;
			};
		};
	
	hashlink **table;				//Storage for all hash pointers

	int hashfunc(string search);
	hashtable(int init_size=1);			//Initial size of hash table is 1.
	void hashcity(city*c);
	void hashlocation(location*L,int counter);
	void linkroads(road*r1,road*r2,string location1, string location2);
	void separatechain(city*c,hashlink*h,int count);
	void addroad(road*r, road*r1, int locnum,int count);
	void set_closest(string closest);
	void read_city(string data,string closest);
	void read_location(string data,int counter);
	void read_road(string data);
	city * findcity(string z);
	int searchloc(string z);
	void resize(string type);
	location * followroad(int roadnum, road*current);

	};
//
//
//
//
//
//-----QUEUE TEMPLATE-----
template <typename T> class queue
	{ 
	protected:

    struct entry
		{ 
		T value;				// Typename specified
		entry * next;			// link to next structure
		entry(T, entry *); 
		};	// constructor

	entry * front, * end;

	public:

    queue();				// constructor
    bool empty();			
    void add(T);
    T take(); 
	};

template <typename T> queue <T> :: entry :: entry(T v, entry * prev)	// constructor for entry
	{ 
	value = v;
	next = prev; 
	}

template <typename T> queue <T> :: queue()		// Constructor for entire queue
	{ 
	front = NULL;
	end = NULL; 
	}
template <typename T> bool queue <T> :: empty()	// Says if queue is empty or not
	{ 
	return end == NULL; 
	}

template <typename T> void queue <T> :: add(T v)
	{ 
	entry * e = new entry(v, NULL);	//Create new entry to add to queue
	if (front == NULL)
		front = e;						//Add to front if the front isn't taken yet
	else	
		end->next = e;					//Add to end otherwise
	end = e;							//Redefine "end" as the newly added entry
	}

template <typename T> T queue <T> :: take()			// Returns object from front of queue and deletes from queue
	{ 
	assert(front != NULL);		// Make sure there is something at the front
	T v = front->value;			// Create temp object set to object at front of queue
	entry * old_front = front;	// Save pointer for old front
	front = front->next;		// Second object is now front
	if (front == NULL)			
		end = NULL;
	delete old_front;
	return v;					// Return object from front
	}
//
//
//
//
//-----CONSTRUCTORS------
hashtable::hashtable(int init_size)
	{
	N=0;
	H=init_size;
	table=new hashlink*[H];
	for(int i=0; i<H; i+=1)
		table[i]=NULL;
	}

city::city(string a,string f1, string f2, string n, string p, string h, string l1, string w1, string l2, string w2, string la, string lo, string cl)
	{
	abbrev=a; fips1=f1; fips2=f2; name=n; population=p; housing=h; land1=l1; water1=w1; land2=l2;
	latitude=la; longitude=lo; closest=cl; 
	}

city::city()					//Constructor for city
	{
	

	}
location::location(string la, string lo, string al, string p, string t, string ab, string n, road * r1)
	{
	latitude = la; longitude = lo; alphadist=al; population = p; type = t; abbrev = ab; name = n; r = r1; distance = 999999;
	}
location::location()
{

}

coverage::coverage(int slat, int elat, int slong, int elong, string f)
{
startlat = slat;
endlat = elat;
startlong = slong;
endlong = elong;
filename = f;
};
coverage::coverage()
{

};

//
//
//
//
//-----MATH-----
string angle(string alphalong, string alphalat, string loclongitude, string loclatitude) //Calculate direction based on two sets of coordinates
	{
	double alatitude = atof(alphalat.c_str());
	double alongitude = atof(alphalong.c_str());
	double loclatnum = atof(loclatitude.c_str());
	double loclongnum = atof(loclongitude.c_str());
	double longdiff = alongitude-loclongnum;
	double latdiff = alatitude-loclatnum;
	double slope = (loclongnum-alongitude)/(loclatnum-alatitude);

	if ((slope<1 || slope>-1.0) && longdiff<0)
		return "W";
	if ((slope<1 || slope>-1.0) && longdiff>0)
		return "E";
	if ((slope>1 || slope<=-1.0) && longdiff<0)
		return "S";
	if ((slope>1 || slope<=-1.0) && longdiff>0)
		return "N";
	else return 
		"?";
	}
//
//
//
//
//------BASIC HASH FUNCTIONS-----
int hashtable::hashfunc(string search)	////Hash function, mulyiplying random number by a hex value with alternating 1's and 0's to maximum randomness.  Setting negatives to positive used instead of unsigned ints.
	{
	int h = 879704;
	for (int i=0; i<search.length(); i+=1)
		h = h * 0xA5A5 + search[i];
	if (h<0)
		h = -h;
	if (h<0)
		h = 982754982;
	return h % H;
	}

void hashtable::resize(string type)
	{
	if (N>H/2)						//Resizes only if over half of spaces used.
		{
		H=H*2;						//Table size doubled.
		hashlink**table2=new hashlink*[H];
		for (int i=0; i<H; i+=1)
			table2[i]=NULL;
		if(type=="cit")				//Resize for alphaplaces, which are hashed.
			{
			for (int i=0; i<H/2; i+=1)
				{
				if(table[i]!=NULL)
					{
					string k=table[i]->data->name;
					table2[hashfunc(k)]=table[i];
					}
				}
			}
		else if (type=="loc")			//Resize for locations, which aren't hashed.
			{	
			for (int j=0; j<H/2; j+=1)
				table2[j]=table[j];
			}
		delete[]table;
		table=table2;
		}
	}

void hashtable::separatechain(city*c,hashlink*h,int count)	////Finds end of chain and adds most recent link of common hash value.  Int count is to keep track of number of collisions per hash value.
	{
	if (h->next==NULL)
		{
		h->next=new hashlink(c,NULL,NULL,NULL);				//Chainging only done alphaplaces, location is NULL
		count=count+1;
		}
	else
		separatechain(c,h->next,count+1);
	}
void hashtable::addroad(road*r,road*r1,int locnum,int count) //Chaining roads to end of chain within location structure
	{
	if (r->next==NULL)
		{
		r->next=r1;
		}
	else if(r->next!=NULL)
		{
		addroad(r->next,r1,locnum,count+1);
		}
	}
//
//
//
//
//-----LINKING FUNCTIONS------
void hashtable::hashcity(city*c)		////Creates a new link at the pointer for the hash value of each city name, or adds a link to a previously filled slot.
	{
	string search = c->name;
	int hashvalue=hashfunc(search);
	if(table[hashvalue]==NULL)
		{
		table [hashvalue]=new hashlink(c,NULL,NULL,NULL);
		}
	else 
		{
		int count=1;
		separatechain(c,table[hashvalue],count);
		}
	N+=1;										//Increment n everytime a city is added.
	string type = "cit";
	resize(type);	
	}
void hashtable::hashlocation(location*L,int counter)
	{
	table[counter] = new hashlink(NULL,L,NULL,NULL);	//The counter is the equivalent of hashvalue for the city, the location is stored linearlly though, it never needs to be searched.
	N+=1;
	string type = "loc";
	resize(type);
	}
void hashtable::linkroads(road*r1,road*r2,string location1, string location2)
	{
	//Road R1 points to location 1.  Road R2 points to location 2.
	//Location1 points to R2, Location2 points to R1
	int locnum1 = atoi(location1.c_str());
	int locnum2 = atoi(location2.c_str());

	if(table[locnum1]->data2->r==NULL)
		{
		table[locnum1]->data2->r=r2;				//Link first location to road
		}
	else if(table[locnum1]->data2->r!=NULL)
		addroad(table[locnum1]->data2->r,r2,locnum1,1);		//Chain road if there is already a road at location
	if(table[locnum2]->data2->r==NULL)
		{
		table[locnum2]->data2->r=r1;					//Link second location to road
		}
	else if(table[locnum2]->data2->r!=NULL)
		addroad(table[locnum2]->data2->r,r1,locnum2,1);		//Chain road if there is already a road at location

	r1->destination=table[locnum1]->data2;
	r2->destination=table[locnum2]->data2;		//linking roads to locations

	}
//
//
//
//
//-----FILE PRINTING-----
void city :: printall(city data)
	{
		cout << "City Name: " << name << "\n" << "State: " << abbrev << "\n" << "FIPS Code: " << fips1 << "\n" << "Place FIPS Code: " << fips2 << "\n" << "Population: " << population << "\n" <<"Housing: " << housing << "\n " << "Land: " << land1 << " square meters(" << land2 << " square miles) \n" << "Water: " << water1 << " square meters\n" << "Coordinates: " <<  latitude << ",  " << longitude << "\n\n";
	}
void location :: printloc()
	{
		cout << "Latitude: " << latitude << "\n" << "Longitude: " << longitude << "\n" << "Distance From nearest city: " << alphadist << "\n" << "Population: " << population << "\n" << "Designation: " << type << "\n" << "Abbreviation: " << abbrev << "\n" << "Name: " << name << "\n\n";
	}
void location :: printallroads(road*r,int count)		//Returns count so search will not search past actual number of roads for invalid input
	{
		location * found = (location*)r->destination;	//Follow road to location at end of road
		globalcount = count+1;
		string s = angle(longitude,latitude,found->longitude,found->latitude);	//Find direction of road
		cout << count << ") " << r->name1 << " heading " << s << "\n";
		if(r->next!=NULL)
			{
			printallroads(r->next,count+1);				//Use counter to keep track of number of roads, access next road in list to keep printing
			}
		cout << "\n";
		
	}
void road :: printroad()
	{
	cout << "Road name: " << name1 << "\n" << "Road Type: " << type << "\nLength: " <<length << "\n\n";
	}
void coverage :: printcov()
	{
	cout << "Starting Latitude: " << startlat << " Ending Latitude: " << endlat << " Starting Longtiude: " << startlong << " Ending Longitude" << endlong << " File Name: " << filename << "\n";
	}
//
//
//
//
//------DRAWING------
void draw_path(location * start, location * end)
	{
	double minlatdeg = (g_lat - g_sec*g_rows)/3600;		// (Total seconds latitude - Seconds per pixel*Number of rows) converted into degrees (1 degree = 3600 seconds)
	double minlongdeg = g_long/3600;
	double maxlatdeg = g_lat/3600;
	double maxlongdeg = (g_long + g_sec*g_col)/3600;	// (Total seconds longitude + Seconds per pixel*number of columns) converted into degrees
	double latdiff = abs(maxlatdeg-minlatdeg);				// Total degrees latitude covered by map	
	double longdiff = abs(minlongdeg-maxlongdeg);				//Total degrees longitude covered by map
	
	double latratio1 = (atof(start->latitude.c_str())-minlatdeg)/latdiff;			//Latitude of first location normalized to fit window divided by total latitude of window to give ratio of position in window vertically
	double longratio1 = abs((atof(start->longitude.c_str())-maxlongdeg)/longdiff);	//Lontitude of first location normalized to fit window divided by total longitude of window to give ratio of position in window horizontally

	double latratio2 = (atof(end->latitude.c_str())-minlatdeg)/latdiff;			
	double longratio2 = abs((atof(end->longitude.c_str())-maxlongdeg)/longdiff);

	move_to(g_col-longratio1*g_col,g_rows-latratio1*g_rows);
	draw_to(g_col-longratio2*g_col,g_rows-latratio2*g_rows);
	}
void city::draw_city (string direction)
	{

	double minlatdeg = (g_lat - g_sec*g_rows)/3600;		
	double minlongdeg = g_long/3600;
	double maxlatdeg = g_lat/3600;
	double maxlongdeg = (g_long + g_sec*g_col)/3600;	
	double latdiff = abs(maxlatdeg-minlatdeg);				
	double longdiff = abs(minlongdeg-maxlongdeg);				
	
	double latratio = (atof(latitude.c_str())-minlatdeg)/latdiff;			
	double longratio = abs((atof(longitude.c_str())-maxlongdeg)/longdiff);
		
	int pop = atoi(population.c_str());

		set_pen_width(8);
		set_font_size(16);
		set_pen_color(0,0,0);
		draw_point(g_col-longratio*g_col,g_rows-latratio*g_rows);			//Points are flipped vertically and horizontally to be positioned correctly.  Longitude ratio multiplied by total columns, latitude ratio multiplied by total rows to fit into window
		move_relative(10,0);
		if (direction=="E")
			write_string(name,direction::east);
		if (direction=="W")
			write_string(name,direction::west);
	}
void draw_country(FILE * f, city *start, city * end)
{

	char dummy[131];
	fread(dummy,131,1,f);  //131 char in dummy line 1 byte each
	std::string metadata(dummy);
	string r, c, lls, tls, spp;

	int pos=0;
	int pos2=0;

	r=metadata.substr(5,4);
	pos=18;
	if(r[3]==' ')
		{
		r=r.substr(0,3);
		pos--;
		}
	c=metadata.substr(pos,4);
	pos = pos+37;
	if(c[3]==' ')
		{
		c=c.substr(0,3);
		pos--;
		}
	spp=metadata.substr(pos,3);
	pos = pos+20;
	if(spp[2]==' ')
		{
		spp=spp.substr(0,2);
		pos--;
		}

	pos2 = pos;
	while(metadata[pos2]!=' ' && pos2<metadata.length())
		pos2++;
	lls=metadata.substr(pos,pos2-pos);
	pos=pos+22;
	pos2=pos;

	while(metadata[pos2]!=' ' && pos2< metadata.length())
		pos2++;
	tls=metadata.substr(pos,pos2-pos);




	g_rows = atoi(r.c_str()); 
	g_col = atoi(c.c_str());
	g_long = atof(lls.c_str());	//Left Long seconds
	g_lat = atof(tls.c_str());		//Top lat Seconds
	g_sec = atof(spp.c_str());
	int pixels = g_rows * g_col;		//total number of data points to be read in based on resolution


	// - READ IN ELEVATION INFO - 

	short int * nextline = new short int[pixels];		//Storage for all altitude values
	int n = fseek(f,2*g_col,0);						//Go to end of dummy line (Based on 2*(R+1)*C equation from notes)
	
	fread(nextline,2,pixels,f);							//Each short int is 2 bytes each
	int largest=0;

	for (int j=0; j<pixels;j++)
	{
		if (nextline[j]>largest)						//Determine what the largest altitude is
			largest=nextline[j];
	}	

	make_window(g_col,g_rows);
	double currentalt;
	double currentcol;

	for (int i=1; i<g_rows; i++)
	{
		for (int j=1; j<g_col; j++)
		{
			currentalt = nextline[i*g_col+j];			//Altitude at the current point
			if (currentalt==-500)
				currentcol = make_color(0,0.7,1.0);		//-500 is special marine number, color point blue
			else
				currentcol = make_color(0,abs(1.0-currentalt/largest),0);		// Read in opposite of current : largest altitude ratio for gradient
			set_pixel_color(j,i,currentcol);
		}
	}

	start->draw_city("W");
	end->draw_city("E");

}
//
//
//
//
//-----FILE READING------
void hashtable::read_city(string data, string closest)	////Breaks each line into substrings which are saved into each respective data value.  Values that may have variable lengths use for loops to avoid reading empty values.
	{
	string a,f1,f2,n,p,h,l1,w1,l2,w2,la,lo,cl;
	
	a = data.substr(0,2);
	f1 = data.substr(2,2);
	f2 = data.substr(4,5);
	n = data.substr(9,63);
	for(int i=0; i<n.length(); i+=1)
		{
		if(n[i]==' ' && n[i+1]==' ')
			{
			n=n.substr(0,i);
			break;
			}
		}

	p = data.substr(73,9);
	for(int i=0; i<p.length(); i+=1)
		{
		if (p[i]!= ' ')
			{
			p=p.substr(i,p.length()-i);
			break;
			}
		}
		
	h = data.substr(82,9);
	for(int i=0; i<h.length(); i+=1)
		{
		if(h[i]!= ' ')
			{
			h=h.substr(i,h.length()-i);
			break;
			}
		}

	l1 = data.substr(91,14);
	for(int i=0; i<l1.length(); i+=1)
		{
		if (l1[i]!= ' ')
			{
			l1=l1.substr(i,l1.length()-i);
			break;
			}
		}

	w1 = data.substr(105,14);
	for(int i=0; i<w1.length(); i+=1)
		{
		if (w1[i]!= ' ')
			{
			w1=w1.substr(i,w1.length()-i);
			break;
			}
		}
	
	l2 = data.substr(119,12);
	for(int i=0; i<l2.length(); i+=1)
		{
		if (l2[i]!= ' ')
			{
			l2=l2.substr(i,l2.length()-i);
			break;
			}
		}
	
	w2 = data.substr(131,12);
	for(int i=0; i<w2.length(); i+=1)
		{
		if (w2[i]!= ' ')
			{
			w2=w2.substr(i,w2.length()-i);
			break;
			}
		}

	la = data.substr(143,10);
	for(int i=0; i<la.length(); i+=1)
		{
		if (la[i]!= ' ')
			{
			la=la.substr(i,la.length()-i);
			break;
			}
		}
	lo = data.substr(153,11);
	for(int i=0; i<lo.length(); i+=1)
		{
		if (lo[i]!= ' ')
			{
			lo=lo.substr(i,lo.length()-i);
			break;
			}
		}
	cl = closest.substr(0,5);
	for(int i=0; i<cl.length(); i+=1)
		{
		if (cl[i]!= ' ')
			{
			cl=cl.substr(i,cl.length()-i);
			break;
			}
		}
	city * temp = new city(a,f1,f2,n,p,h,l1,w1,l2,w2,la,lo,cl);		//Reads values into a new data structure.
	hashcity(temp);							//Assigns a pointer of the hash value of the city name for each data structure.
	}
void hashtable::read_location(string data,int counter)
	{	
	string la, lo, al, p, t, ab, n;

	lo = data.substr(0,9);
	for(int i=0; i<lo.length(); i+=1)
		{
		if (lo[i]!= ' ')
			{
			lo=lo.substr(i,lo.length()-i);
			break;
			}
		}
	la = data.substr(12,7);
	for(int i=0; i<la.length(); i+=1)
		{
		if (la[i]!= ' ')
			{
			la=la.substr(i,la.length()-i);
			break;
			}
		}
	al = data.substr(20,5);
	for(int i=0; i<al.length(); i+=1)
	{
	if (al[i]!= ' ')
		{
		al=al.substr(i,al.length()-i);
		break;
		}
	}
	p = data.substr(28,7);
	for(int i=0; i<p.length(); i+=1)
		{
		if (p[i]!= ' ')
			{
			p=p.substr(i,p.length()-i);
			break;
			}
		}
	t = data.substr(35,4);
	ab = data.substr(40,2);
	n = data.substr(43,63);
	for(int i=0; i<n.length(); i+=1)
		{
		if(n[i]==' ' && n[i+1]==' ')
			{
			n=n.substr(0,i);
			break;
			}
		}
	location * temp = new location(la, lo, al, p, t, ab, n, NULL);
	hashlocation(temp,counter);
	}
void hashtable::read_road(string data)
	{
	string n,t,l1,l2,len;
	int pos = 0;
	while(data[pos]!=' ' && pos<data.length())
		pos++;
	n=data.substr(0,pos);
	pos++;
	int pos2=pos;
	while(data[pos2]!=' ' && pos2<data.length())
		pos2++;
	t=data.substr(pos,pos2-pos);
	pos2++;
	pos=pos2;
	while(data[pos2]!=' ' && pos2<data.length())
		pos2++;
	l1=data.substr(pos,pos2-pos);					
	pos2++;											
	pos=pos2;												
	while(data[pos2]!=' ' && pos2<data.length())	
		pos2++;										
	l2=data.substr(pos,pos2-pos);						
	pos2++;												
	pos=pos2;											
	while(data[pos2]!=' ' && pos2<data.length())		
		pos2++;											

	len=data.substr(pos,pos2-pos);

	road * temp1 = new road(n,t,len,NULL,NULL);
	road * temp2 = new road(n,t,len,NULL,NULL);
	linkroads(temp1,temp2,l1,l2);

	}
coverage read_coverage(string data)
	{
	string slat, elat, slong, elong, f;

	int pos = 0;
	int pos2 = 0;
	
	slat = data.substr(0,2);
	elat = data.substr(3,2);
	pos = 6;						//Starting position for starting longitude if endling latitude is two character
	if (elat[1]==' ')
		{
		elat=elat.substr(0,1);		//Ending latitude is either one or two digits
		pos = 5;					//Starting position for starting longitude if ending latitude is one character
		}

	slong = data.substr(pos,4);
	pos = pos+5;
	if (slong[3]==' ')				//Starting longitude is either 3 or 4 characters
		{
		slong = slong.substr(0,3);
		pos--;
		}
	
	elong = data.substr(pos,4);		//Endling longitude is also either 3 or 4 characters
	pos = pos+5;
	if (elong[3]==' ')
		{
		elong = elong.substr(0,3);
		pos--;
		}

	f = data.substr(pos,17);
	for(int i=0; i<f.length(); i+=1)
		{
		if(f[i]==' ')
			{
			f=f.substr(0,i);
			break;
			}
		}
	int slatnum=atoi(slat.c_str());
	int elatnum=atoi(elat.c_str());
	int slongnum=atoi(slong.c_str());
	int elongnum=atoi(elong.c_str());
	coverage temp(slatnum,elatnum,slongnum,elongnum,f);
	return temp;

}
//
//
//
//
//-----SEARCHING------
location * hashtable::followroad(int roadnum,road*current)		//Find location at the end of a road
{	
	if (roadnum>0)
		followroad(roadnum-1,current->next);
	location * temp = (location*)current->destination;	
	return temp;

}
city * hashtable::findcity(string z)
	{
	if(table[hashfunc(z)]==NULL && z!="exit" && z!="Exit" && z!="EXIT")	//Will not print "Not found" message when exiting program.
		{
		cout << "Not found";
		return NULL;
		}

	hashlink * look = table[hashfunc(z)];
	city * temp = new city[50];				// Storage of multiple cities of same name
	int counter = 0;

	while(look!=NULL)
		{
		if (look->data->name==z)  			//Only print objects of the same hash value if the names match.
			{
			temp[counter] = *(look->data);
			}
		look=look->next;
		counter++;
		}
	return temp;
	}	


string smallestmap(location * start, location * end, coverage * all)
	{
	bool latrange=false;		// Locations fall into latitude range of current window
	bool longrange=false;		// Locations fall into longitude range of current window
	int winsize = 5;			// Start with smallest range of coverage
	
	double startlatnum = atof(start->latitude.c_str());
	double startlongnum = atof(start->longitude.c_str());
	double endlatnum = atof(end->latitude.c_str());
	double endlongnum = atof(end->longitude.c_str());

	while(winsize<55)			//Increment coverage size up until maximum of 50
		{
		for (int i=0; i<149; i++)
			{
			if (all[i].startlat - all[i].endlat == winsize)		// Only compare for appropriate range
				{
				if (startlatnum < all[i].startlat && endlatnum < all[i].startlat && startlatnum > all[i].endlat && endlatnum > all[i].endlat) //Both points fit into latitude range of window
					latrange = true;
				else latrange = false;
				if (startlongnum < all[i].endlong && endlongnum < all[i].endlong && startlongnum > all[i].startlong && endlongnum > all[i].startlong) //Both points fit into longitude range of window
					longrange = true;
				else longrange = false;
				}
			if (latrange==true && longrange==true)
				return all[i].filename;			//If beginning and ending point fit into range, return the name of the window;
			}
		winsize +=5;
		}
	string fail = "Coverage failed";
	return fail;						// Return message if function did not work
	}


void consider(location*&A,queue<location*>&s,double when)
	{
	if (A!=NULL && A->distance > when)		// Current distance is less than distance already saved
		{
		A->distance=when;					// Save new shortest path
		s.add(A);							// Add object to queue
		}
	}
void retrace(location * start, location * end)
	{
	location * temp = end;				//Set current location to finishing point
	road * z = end->r;					//Choose road as first pointer
	double mindist = 99999;				//For choosing shortest route back to starting point
	int count=0;
	string savename;					//
	double savelength=0;				//These are for correctly writing directions later
	while(temp!=start)
		{
		while(z!=NULL)
			{
			location * dest = (location*)z->destination;	//Location at end of current road
			double roadlen = atof(z->length.c_str());
			if(roadlen+dest->distance<mindist)				//Destination's length from start + road length
				mindist = roadlen+dest->distance;			//Set new smallest distance out of choices for roads
			z=z->next;
			}
		z=temp->r;											//reset z to run through available roads again;
		while(z!=NULL)
			{
			location * dest = (location*)z->destination;
			double roadlen = atof(z->length.c_str());
			if(roadlen+dest->distance==mindist)
				{
				set_pen_color(color::red);								
				set_pen_width(2);
				draw_path(temp,dest);
				if (savename==z->name1)
					savelength=savelength+roadlen;													//Increment this int if staying on the same road at an intesection
				if(z->name1!=savename && savelength!=0)
					cout << "Drive " << savelength*69 << " miles on " << z->name1 << "\n";			//Only print if the road you're on changes
				else if (savelength==0)
					cout << "Drive " << roadlen*69 << " miles on " << z->name1 << "\n";				//For first time when savelength initialized to 0
				if (savename!=z->name1) 
					savelength = roadlen;															//Reset saved length if road changes
				savename = z->name1;
				temp=dest;
				break;																				//Break from loop once a road is traced
				}
			if (z->next!=NULL)
				z=z->next;
			}
		z=temp->r;
		}
	}
void shortestpath(location * start, location * end)
	{
	queue <location*> s;					// Creates queue
	location * A = start;					// Starting point
	s.add(A);								// Add starting point to the queue
	A->distance=0;							// Initialize distance to zero
	while(!s.empty())
		{
		location * X = s.take();			// Will take starting point on first run through.
		double distnow = X->distance;		// Reads current distance
		road * z = X->r;					// First road of current location
		int count =0;
		while(z!=NULL)
			{
			double rlength = atof(z->length.c_str());		// length of current road
			location * dest = (location*)z->destination;	// Destination at end of road must be converted from void pointer
			consider(dest,s,distnow+rlength);				//
			z=z->next;										// Explore next road on path
			}
		}
	printf("\n");
	cout << "\nDONE\n";
	retrace(start,end);
	}
//
//
//
//
//-----MAIN-----
void main ()
{
	//*****READ ALL INFORMATION*****
	hashtable * alllocation = new hashtable;		//Storage of all locations
	hashtable * allcity = new hashtable;			//Storage of all cities
	coverage * allcoverage = new coverage[149];		//149 total maps to be saved

	ifstream fin2("locations.txt");
	int counter = 0;	
	while(true)			//Read locations text.
		{
		string line2;
		getline(fin2,line2);
		if(fin2.fail())
			{	
			cout <<"Locations read";
			break;
			}
		alllocation->read_location(line2,counter);
		counter++;	//This is neccesary for putting location into array position linearlly because locations aren't hashed.
		}
	

	ifstream fin("alphaplaces.txt");
	ifstream fin3("closests.txt");
	while(true)			//Read alphaplaces and closests texts.				
		{

		string line;
		string line3;
		getline(fin,line);	
		getline(fin3,line3);
		if (fin.fail())
			{	
			break;			
			}
		if (fin3.fail())
			{	
			cout << "\n Alphaplaces amd closests read";
			break;			
			}
		allcity -> read_city(line,line3);
		}
	

	
	ifstream fin4("majorroads.txt");
	while(true)			//Read roads text			
		{
		string line4;
		getline(fin4,line4);
		if (fin4.fail())
			{	
			cout << "\n Roads read";
			break;			
			}
		alllocation -> read_road(line4);
		}

	int count1=0;
	ifstream fin5("coverage.txt");
	while(true)
		{
		string line5;
		getline(fin5,line5);
		if (fin5.fail())
			{
			cout << " \nCoverage reading complete\n";
			break;
			}
		allcoverage[count1]=read_coverage(line5);
		count1++;
		}

	//*****USER INTERFACE******

	string t;										
	string abbreviation;							// Start city name & state abbrev
	string t2;
	string abbrev2;									// End city name & state abbrev

	int closestloc = 0;								
	int closestloc2 = 0;								// Array positions for starting and ending locations


		cout <<	"\nEnter destination city, or exit to quit: ";
		getline(cin,t);		
		cout << "Enter state abbreviation: ";
		getline(cin,abbreviation);					

		city * startcity = allcity->findcity(t);	//All cities with matching name will be saved in the array.  Only ones with the proper abbreviation will be accessed
		int count =0;
		if (startcity==NULL)
			found1 = false;
		else
			{
			while(startcity[count].name!="" && startcity!=NULL)				//Return all cities with given details
				{
				if (startcity[count].abbrev==abbreviation)
					{
					startcity[count].printall(startcity[count]);			//Print proper city
					closestloc = atoi(startcity[count].closest.c_str());	//Position in location array of closest location to city
					string s=angle(startcity[count].longitude,startcity[count].latitude,alllocation->table[closestloc]->data2->longitude,alllocation->table[closestloc]->data2->latitude);	//Alpha Longitude, Alpha Latitude, Location Longitude, Location Latitude
					cout << "Intersection is " << alllocation->table[closestloc]->data2->alphadist << " miles " << s << " of " << startcity[count].name << "\n\n";	

					road * z = alllocation->table[closestloc]->data2->r;	//Pointer to list of roads leading from closest location
					found1 = true;
					}
				count++;
				}
			}

		cout << "\nEnter starting point: ";
		getline(cin,t2);		
		cout << "Enter state abbreviation of starting point: ";
		getline(cin,abbrev2);	

		city * endcity = allcity->findcity(t2);
		int count2 = 0;
		if (endcity==NULL)
			found1 = false;
		else
			{
			while(endcity[count2].name!="" && endcity!=NULL)				//Return all cities with given details
				{
				if (endcity[count2].abbrev==abbrev2)
					{
					endcity[count2].printall(endcity[count2]);			//Print proper city
					closestloc2 = atoi(endcity[count2].closest.c_str());	//Position in location array of closest location to city
					string s2 = angle(endcity[count2].longitude,endcity[count2].latitude,alllocation->table[closestloc2]->data2->longitude,alllocation->table[closestloc2]->data2->latitude);	//Alpha Longitude, Alpha Latitude, Location Longitude, Location Latitude
					cout << "Intersection is " << alllocation->table[closestloc2]->data2->alphadist << " miles " << s2 << " of " << endcity[count].name << "\n\n";	

					road * z2 = alllocation->table[closestloc2]->data2->r;	//Pointer to list of roads leading from closest location
					found1 = true;
					
					alllocation->table[closestloc]->data2->destination = alllocation->table[closestloc2]->data2; //Set destination of first location as the second destination
					}
				count2++;
				}
			}

	if(endcity!=NULL && startcity!=NULL)
	{
	string smallest = smallestmap(alllocation->table[closestloc]->data2, alllocation->table[closestloc2]->data2,allcoverage);


	//****DRAW COUNTRY****

	FILE * f = fopen(smallest.c_str(), "rb");	// read binary
	if (f==NULL)
		printf("Your file sucks brah");
	
	draw_country(f,startcity,endcity);
	fclose(f);	


	shortestpath(alllocation->table[closestloc]->data2,alllocation->table[closestloc2]->data2);
	}

}

